<?php
namespace Neuron\Person\Controller\Api;

class Profile extends \Neuron\Person\Controller {
    const SUCCESS_CODE = 0;
    const FAIL_CODE = 1;
    const CODE_FIELD_REQUIRED  = 400;
    const CODE_UNAUTHORIZED = 401;
    const CODE_SESSION_EXPIRED = 403;

    public function selfDataAction(){
        $result = new \Neuron\Generic\Result();
        
        $request = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
            
        }
        $data = $this->getREST()->getParameters();
        $username = isset($data['username'])?$data['username']:'';
        $token = isset($data['token'])? $data['token']: '';
        // print_r($session_id); die();
        try {
            if($token == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }
            
            $session =$this->getSession();

            $storage = \Neuron\Composite\User\Storage::factory($this->getDb(), $this->getConfig());
            $user = new \Neuron\Composite\User($storage, $this->getConfig()['user']);

            /* Load user by user code */
            $user->load($username, \Neuron\Actor\User\Storage::LOADBY_CODE);

            $storagePerson = \Neuron\Person\Entity\Storage::factory($this->getDb(), $this->getConfig());
            $cpersons = new \Neuron\Person\Entity($storagePerson);
    
            $mapPerson = $cpersons->loadMap($user->id,'user_id')['data'];
            // print_r($mapPerson); die();

            $session->owner($user->id);

            
            // $sessid = $usrSession->getSessionId($userId);
            $sessid = $session->getId();
            if($token != $sessid){
                $result->code = self::CODE_UNAUTHORIZED;
                $result->info = 'Failed authenticate';
                return $this->getREST()->getResponse($result);   
            }

            $islogin = $this->isLoggedIn();
            if(!$islogin){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result);   
            }

            $cpersons->id = $mapPerson['person_id'];

            // $storage0 = \Neuron\HRMIS\Attribute\Storage::factory($this->getDb());
            // $attribute = new \Neuron\HRMIS\Attribute($storage0);

            // $attr = $attribute->loadEmpAttr(1, 1, $prof->employee_id);
            $attrs = array();
            // if (!empty($attr)) {
            //     foreach ($attr as $val) {
            //         array_push($attrs, $val);
            //     }
            // }

            $profile = $cpersons->loadPersonDetail($mapPerson['person_id']);
            if($profile['pic_official']){
                if(!file_exists(getcwd() .'/public'. $profile['pic_official'])){
                    $profile['pic_official'] =  $this->getPublicUrl().'/actor/picture/default';
                }else{
                    $profile['pic_official'] = $this->getPublicUrl() . "" . $profile['pic_official'];
                }
            }else{
                $profile['pic_official'] = $this->getPublicUrl().'/actor/picture/default';
            }
            $profile['pic_signature'] = $this->getPublicURL().$profile['pic_signature'];
        
            $contact = $this->getDataFromUrl('person_contact', '{"username":"'.$username.'", "token":"'.$token.'", "person_id":"'.$mapPerson["person_id"].'"}');
    
            // if ($cont) {
            //     foreach ($cont as $conts) {
            //         array_push($contact, $conts);
            //     }
            // }

            $dataRes = array('profile'=>$profile,'contact'=>$contact['contact'], 'attributes'=>$attrs);
            $result->guid = self::SUCCESS_CODE;
            $result->code = self::SUCCESS_CODE;
            $result->info = 'OK';
            $result->data = $dataRes;
            return $this->getREST()->getResponse($result);

        } catch (Exception $e) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }

    public function saveProfileAction() {
        $result = new \Neuron\Generic\Result();
        
        $request = $this->getRequest();

        if (!$request->isPost()) {
            $result->code = self::FAIL_CODE;
            $result->info = 'Method not allowed';
            return $this->getREST()->getResponse($result);
            
        }
        $data = $this->getREST()->getParameters();
        $userId = isset($data['username'])?$data['username']:'';
        $token = isset($data['token'])? $data['token']: '';
        $params = isset($data['params'])? $data['params']:null;

        try {
            if($userId == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'username is null';
                return $this->getREST()->getResponse($result);   
            }

            if($token == ""){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'token is null';
                return $this->getREST()->getResponse($result);   
            }

            if($params == NULL){
                $result->code = self::CODE_FIELD_REQUIRED;
                $result->info = 'params is null';
                return $this->getREST()->getResponse($result);   
            }

            $storage = \Modules\Users\User\Storage::factory($this->getDb());
            $usrSession = new \Modules\Users\User\Session($storage);
            
            $session =$this->getSession();
            $session->owner($userId);

            // $sessid = $usrSession->getSessionId($userId);
            $sessid = $session->getManager()->getId();
            if($token != $sessid){
                $result->code = self::CODE_UNAUTHORIZED;
                $result->info = 'Failed authenticate';
                return $this->getREST()->getResponse($result);   
            }

            $islogin = $this->isLoggedIn();
            if(!$islogin){
                $result->code = self::CODE_SESSION_EXPIRED;
                $result->info = 'Session Expired';
                return $this->getREST()->getResponse($result);   
            }

            $storage = \Neuron\HRMIS\Profile\Storage::factory($this->getDb());
            $prof = new \Neuron\HRMIS\Profile($storage);

            $prof->employee_id = $userId;

            foreach ($params as $tosave) {
                if(array_key_exists("attr_id", $tosave) && $tosave["attr_id"]){
                    $storage = \Neuron\HRMIS\Attribute\Storage::factory($this->getDb());
                    $attribute = new \Neuron\HRMIS\Attribute($storage);


                    $save = $attribute->saveEmpAttr(($tosave['emp_attr_id'] != '' ? 'update' : 'create'), $tosave['attr_type_id'], $userId, $tosave['attr_type_id'], $tosave['value']);
                    if(!$save) {
                        $dataRes = array('success'=> false, 'message'=>'Error on save profile attribute');
                        $result->guid = self::SUCCESS_CODE;
                        $result->code = self::FAIL_CODE;
                        $result->info = 'Error on save profile attribute';
                        $result->data = $dataRes;
                        return $this->getREST()->getResponse($result);
                    }
                }else{
                    $prof->form_name = $tosave['name'];
                    $prof->form_value = $tosave['value'];

                    if ($tosave['name'] == 'birth_date' || $tosave['name'] == 'join_date') {
                        $prof->form_value = date('Y-m-d', strtotime($tosave['value']));
                    }
                    $update = false;
                    if ($userId != '')
                        $update = true;

                    $save = $prof->save($update);
                    if(!$save) {
                        $dataRes = array('success'=> false, 'message'=>'Error on save profile');
                        $result->guid = self::SUCCESS_CODE;
                        $result->code = self::FAIL_CODE;
                        $result->info = 'Error on save profile';
                        $result->data = $dataRes;
                        return $this->getREST()->getResponse($result);
                    }
                }
            }

            if ($save) {
                $dataRes = array('success'=> true, 'message'=>'Data saved!');

                $result->guid = self::SUCCESS_CODE;
                $result->code = self::SUCCESS_CODE;
                $result->info = 'Data saved!';
                $result->data = $dataRes;
                return $this->getREST()->getResponse($result);
            } else {
                $dataRes = array('success'=> false, 'message'=>'Error on save profile');
                
                $result->guid = self::SUCCESS_CODE;
                $result->code = self::FAIL_CODE;
                $result->info = 'Error on save profile';
                $result->data = $dataRes;
                return $this->getREST()->getResponse($result);
            }
        } catch (Exception $e) {
            $result->code = self::FAIL_CODE;
            $result->info = 'error';
            $result->data = array(
                'message' => $ex->getMessage(),
            );
            return $this->getREST()->getResponse($result);
        }
    }
}