<?php

namespace Neuron\Person\Controller;

use Neuron\Person\Skills;
use Neuron\Person\Skill as mSkill;
use Neuron\Person\Entity;
use Neuron\Generic\Result;

class Skill extends \Neuron\Person\Controller
{
	protected function getView($name, $variables = array())
	{
		/* default layout & view prefix */
		$layout = 'neuron/person/layout/';
		$prefix = 'neuron/person';
		if (!class_exists('Neuron\Application\Framework\Entity')) {
			/* libraries */
			$this->using('jquery', null)
				->using(
					'bootstrap-4',
					null,
					array(
						'https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js',
						'https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js',
						'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css',
						'https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js'
					)
				)
				->using('fontawesome', null);
		}
		/* get layout n view prefix from config */
		$config = $this->getConfig();
		if (isset($config['person'])) {
			$config = $config['person'];
			$layout = isset($config['formlayout']) ? $config['formlayout'] : $layout;
			$prefix = isset($config['viewprefix']) ? $config['viewprefix'] : $prefix;
		}

		$this->using('nAlert', 9999999);

		/* set layout and view */
		$this->layout($layout);
		$view = new \Zend\View\Model\ViewModel($variables);
		$view->setTemplate($prefix . '/' . strtolower($name));

		return $view;
	}

	public function listAction()
	{
		$layout = $this->layout();
        $layout->setVariables(['current' => 'Manage Skill']);

		$this->using('datatable', 19, array(
			'https://cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js',
			'https://cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js',
			'https://cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css'
		));
		$this->usingJs('bootbox',21,  array(
			'https://cdnjs.cloudflare.com/ajax/libs/bootbox.js/4.4.0/bootbox.min.js'
		));

		$this->using('p-loading', 22);
		$this->usingJs('moment', 23, array(
			'https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.29.1/moment.min.js'
		));
		$this->using('bootstrap-datepicker', 24);
		$this->using('select2', 26, array(
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/select2.min.js',
			/* ini load sesuai bahasa dari session */
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/js/i18n/id.js',
			/**/
			'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.10/css/select2.min.css',
			'https://cdn.jsdelivr.net/npm/@ttskch/select2-bootstrap4-theme@1.3.2/dist/select2-bootstrap4.min.css',
		));
		$this->using('generic', 30);
		$this->using('jquery.validate', 31, array(
			"https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/jquery.validate.min.js",
			"https://cdnjs.cloudflare.com/ajax/libs/jquery-validate/1.19.2/additional-methods.min.js"
		));

		$title 	= $this->translate("Manage Skill");
        $post   = $this->getRequest()->getQuery();
		$id		= isset($post['id']) ? $post['id'] : null;
		$entity	= new Entity(Entity\Storage::factory($this->getDb(),$this->getConfig()));
		$entity->load($id, Entity\Storage::LOADBY_ID);
		$person = $entity->first_name.' '.$entity->last_name;

		return $this->getView('skill/list', get_defined_vars());
	}

	public function loadListAction()
	{
        $post	= $this->getPostParams();
        
        $skill  = new Skills(mSkill\Storage::factory($this->getDb(),$this->getConfig()));
        if (isset($post['start']) && isset($post['length'])) {
            $params = array(
                'page' 		=> ($post['start'] + $post['length']) / $post['length'],
                'records' 	=> $post['length'],
                'order' 	=> $post['order'],
                'columns' 	=> $post['columns'],
				'person_id'	=> $post['id']
            );

            $keyword 	= $post['search']['value'];
            $datatable 	= true;
            if ($post['length'] == -1) {
                $mode 	= \Neuron\Person\Skill\Storage::LIST_ALL;
            } else {
                $mode 	= \Neuron\Person\Skill\Storage::LISTBY_PAGE;
            }
        } else {
            $post 		= $this->getREST()->getParameters();
            $params 	= is_array(@$post['additional']) ? @$post['additional'] : array();
            $keyword 	= @$post['keyword'];
            $mode 		= isset($post['mode'])? $post['mode']: \Neuron\Person\Skill\Storage::LIST_ALL;
            $datatable 	= false;
        }

        $result = $skill->load($keyword, $mode, $params);
        
        if ($datatable) {
            if ($result->code == $result::CODE_SUCCESS) {
                $listData['draw'] = $post['draw'];
                $listData['recordsTotal'] = $result->data['countTotal'];
                $listData['recordsFiltered'] = $result->data['countTotal'];
                $listData['data'] = @$result->data['list'];
            } else {
                $listData['draw'] = $post['draw'];
                $listData['recordsFiltered'] = 0;
                $listData['recordsTotal'] = 0;
                $listData['data'] = array();
            }
            
            return $this->getOutput(json_encode($listData));
        }

  		return $this->getRest()->getResponse($result);
    }

	public function saveAction(){
		$sessmgt 		= $this->getSession()->owner();
		$data       	= $this->getRequest()->getPost();
        if (empty($data)) {
            $data   	= $this->getREST()->getParameters();            
        }

		$result			= new Result();

		$data['start_date']	= date('Y-m-d');
		$data['update_dtm']	= date('Y-m-d H:m:s');
		$data['update_by']	= $sessmgt;

		$skill 			= new mSkill(mSkill\Storage::factory($this->getDb(), $this->getConfig()));
		$skill->load(null, mSkill\Storage::LOADBY_PARAMS, $data);

		if (empty($data['person_skill_id'])) {
			if ($skill->person_skill_id) {
				$result->code = 401;
				$result->info = $this->translate('skill already exists');
	
				return $this->getREST()->getResponse($result);
			}
		}
		
		$result 		= $skill->create($data);
		$result->info 	= $this->translate($result->info);

		return $this->getREST()->getResponse($result);
	}

	public function deleteAction(){
		$data       	= $this->getRequest()->getPost();
        if (empty($data)) {
            $data   	= $this->getREST()->getParameters();            
        }

		$skill 			= new mSkill(mSkill\Storage::factory($this->getDb(), $this->getConfig()));
		$result 		= $skill->delete($data['person_skill_id']);
		$result->info 	= $this->translate($result->info);

		return $this->getREST()->getResponse($result);

	}
}

