<?php
namespace Neuron\Person;

use Neuron\Generic\Entity\Map;
use Neuron\Generic\Result;

class Detail extends \Neuron\Generic\Entity\Mapper {
    
    protected $_storage;
    const ERROR_PREFIX  = 'PERSON_DETAIL-';

    public function __construct(\Neuron\Person\Detail\Storage\Skeleton $storage, $data = array()) {
        $this->_storage = $storage;
        parent::__construct(
            $data, 
            array(
                new Map('detail_id'),
                new Map('person_id'),
                new Map('pic_official'),
                new Map('pic_signature'),
                new Map('about'),
                new Map('flag_geolocation'),
                new Map('flag_attd_evidence'),
                new Map('termination_reason'),
                new Map('termination_reason_category'),
                new Map('evidence_id'),
                new Map('flag_timetable_overlap'),
                new Map('flag_bpjs_ks'),
                new Map('flag_bpjs_kt'),
            ), 
            parent::CASE_INSENSITIVE
        );
    }
	
    public function load($id, $mode = \Neuron\Person\Detail\Storage::LOADBY_ID) {
        if ($data = $this->_storage->load($id, $mode)) {
            @$this->push($data);
            return true;
        }
        return false;
    }

    public function save($update = false){
        if ($id = $this->_storage->save($this, $update)) {
            $this->detail_id = $id;
            return true;
        }
        return false;
    }

    public function saveEx($data){
        $result     = new Result();
        $validate = $this->validate($data);
        if($validate->code != 0){
            return $validate;
        }
        if(isset($data['detail_id']) && !empty($data['detail_id'])){
            $update = true;
        }else{
            $update = false;
        }
        $this->push($data);
        if(!$this->save($update)){
            $result->code = 1;
            $result->info = "Failed saving detail";
            return $result;
        }
        $result->data   = $this->pull();
        return $result;
    }


    public function validate(){
        $result     = new Result();
        /* Validate detail */
        return $result;
    }

    public function delete($id) {
        return $this->_storage->remove($id);
    }
    
}