<?php
namespace Neuron\Person;

use Neuron\Generic\Entity\Map as EntityMap;
use Neuron\Generic\Result;
use Neuron\Person\Traits\Model;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;

class Map extends \Neuron\Generic\Entity\Mapper {
    use Model;

    CONST ERROR_PREFIX      = 'PER-MAP-';

    protected $_storage;

    public function __construct(\Neuron\Person\Map\Storage\Skeleton $storage, $data = array()) {

        $this->_storage = $storage;

        parent::__construct(
            $data, 
            array(
                new EntityMap('user_map_id'),
                new EntityMap('person_id'),
                new EntityMap('user_id'),
                new EntityMap('start_date'),
                new EntityMap('end_date')
            ), 
            parent::CASE_INSENSITIVE
        );
    }

    public function person(){
        $entity     = new Entity(Entity\Storage::factory($this->getDb(),$this->getConfig()));
        $entity->load($this->person_id);
        return $entity;
    }
	
	public function load($id = null, $mode = \Neuron\Person\Map\Storage::LOADBY_ID,$params = array()) {
		if ($data = $this->_storage->load($id,$mode, $params)) {
            $this->push($data);
            return true;
		}
		return false;
	}

    public function save($update = false){
		if ($id = $this->_storage->save($this, $update)) {
		  $this->user_map_id = $id;
		  return true;
		}
		return false;
    }

    public function create($data = array())
    {        
		$result = new Result();
		$update = false;
		if($data['user_map_id']){
			$update = true;
		}

		//create data relation
		$param = $this->pull();
		foreach($data as $k => $v){
			if(array_key_exists($k, $param)){
				$param[$k] = $v;
			}
		}

		$this->push($param);
		if($this->save($update)){
			$result->data = $this->user_map_id;
		}
		return $result;
    }

    public function delete($id) {
        return $this->_storage->remove($id);
    }

}