<?php
namespace Neuron\Person;

use Neuron\Application\Framework as NAF;
use Neuron\Application\Framework\Hook\Manager;
use Neuron\Actor\User\Authentication;
use Neuron\Application\Framework\Entity;
use Neuron\Generic\Result;
use Neuron\Notification\Profile;
use Neuron\Person\Entity as mEntity;
use Neuron\Person\Entities as mEntities;

class Module {
    const CODE_FAIL = 1;
    const DEFAULT_LANGUAGE = 'id_ID';
    const DEFAULT_PERMISSION = 4;
    const BASE_URL_MASTER = 'https://route.doorhrm.com/'; 
    const API_LOAD_COMPANY = 'api/company/loadby';
    const API_USER_CREATE = 'api/user/create';

    public function __construct() {
        $manager = Manager::get();
        $manager->events()->register('naf_module_install', array($this, 'onModuleInstall'));
        $manager->events()->register('naf_module_uninstall', array($this, 'onModuleUninstall'));
        $manager->events()->register('naf_module_update', array($this, 'onModuleUpdate'), 1, 2);
        $manager->events()->register('person_create_user', array($this, 'createUser'), 10, 2);

        $manager->events()->register('actor_profile_nav_before', array($this, 'addProfileBeforePills'), 1, 1);
        $manager->events()->register('actor_profile_nav_after', array($this, 'addProfileAfterPills'), 1, 1);
        $manager->events()->register('actor_profile_content', array($this, 'addContentPersonal'), 1,1);
        $manager->events()->register('actor_button_under_avatar', array($this, 'downloadCv'),10,2);
        // $manager->events()->register('actor_profile_content', array($this, 'addContentExperience'), 1,1);
        // $manager->events()->register('actor_profile_content', array($this, 'addContentEducation'), 1,1);
        // $manager->events()->register('actor_profile_content', array($this, 'addContentMedical'), 1,1);
        // $manager->events()->register("theme_profile_url", [$this,'setEmployeeProfileURL'],10,2);
        $manager->events()->chain('naf_search', [$this, 'nafDoSearch'], 0, 1);
    }

    public function setEmployeeProfileURL($basePath, $user) {
        return $basePath . '/person/index/detail';
    }

    public function onModuleInstall() {

        /* get NAF entity */
        if ($naf = NAF\Entity::get()) {

            /* get dml file */
            if (strtolower($naf->database()->getType()) == 'oracle') {
                $file = __DIR__ . '/Resource/Db/Oci8.php';
            } else {
                $file = __DIR__ . '/Resource/Db/Mysql.php';
            }

            /* include it */
            $sqls = include($file);
            if (is_array($sqls)) {
                foreach ($sqls as $sql) {
                    $naf->database()->execute($sql);
                }
            }

            /* register menu for this library */
            $naf->menu()->add('mnu_person', null, 'Person',null,'PERSON');
            $naf->menu()->add('mnu_person_status', '/person/index/status', 'Status', 'mnu_person',"PERSON");
            $naf->menu()->add('mnu_person_type', '/person/index/type', 'Type', 'mnu_person',"PERSON");
            $naf->menu()->add('mnu_person_main', '/person/index/manage', 'Manage Person', 'mnu_person',"PERSON");
            $naf->menu()->add('mnu_person_attribute', '/person/attribute-type', 'Person Attribute', 'mnu_person',"PERSON");
            $naf->menu()->add('mnu_person_upload', '/person/index/upload', 'Upload Person', 'mnu_person',"PERSON");
            $naf->menu()->add('mnu_person_skill_type', '/person/skill-type/list', 'Parameters SKill Type', 'mnu_person',"PERSON");
            
            
            /* add js libs */
            $this->downloadAsset("https://git.neuron.id/pub/lib/raw/master/personasset.zip", "personasset.zip", "/public/userfiles/person");

            $this->updateNotificationProfile();
            
        }
    } //omi

    public function onModuleUninstall() {

        /* get NAF entity */
        if ($naf = NAF\Entity::get()) {

            /* deregister menu for this library */
            $naf->menu()->remove('mnu_person');
            $naf->menu()->remove('mnu_person_manage');
            $naf->menu()->remove('mnu_person_param');
            $naf->menu()->remove('mnu_person_attribute');
            $naf->menu()->remove('mnu_person_skill_type');

        }
    }  //omu

    public function onModuleUpdate($new,$old){

        if($new == '202-reporting-turnover') $new = 'v1.5.45';
        
        if($naf = \Neuron\Application\Framework\Entity::get()){

			if(version_compare($old,'v1.5.12') == -1){
				$sql 	= include(__DIR__.'/Resource/Db/v1.5.12/Mysql.php');
				foreach($sql as $sql){
					$naf->database()->execute($sql);
				}
            }
            
			if(version_compare($old,'v1.5.15') == -1){
				$sql 	= include(__DIR__.'/Resource/Db/v1.5.15/Mysql.php');
				foreach($sql as $sql){
					$naf->database()->execute($sql);
				}
            }

            // untuk ubah person_type.id jadi auto increment
            $rs = array();
            if(version_compare($old,'v1.5.19') == -1){
				$sql 	= include(__DIR__.'/Resource/Db/v1.5.19/Mysql.php');
				foreach($sql as $sql){
					$rs[] = $naf->database()->execute($sql);
				}
            }

            // penambahan field turnover_reason & evidence_id di person detail
			if(version_compare($old,'v1.5.45') == -1){
				$sql 	= include(__DIR__.'/Resource/Db/v1.5.45/Mysql.php');
				foreach($sql as $sql){
					$naf->database()->execute($sql);
				}
            }

            if(version_compare($old,'v1.5.47') == -1){
				$sql 	= include(__DIR__.'/Resource/Db/v1.5.47/Mysql.php');
				foreach($sql as $sql){
					$naf->database()->execute($sql);
				}
            }

            if(version_compare($old,'v1.5.60') == -1){
                $sql 	= include(__DIR__.'/Resource/Db/v1.5.60/Mysql.php');
				foreach($sql as $sql){
					$rs[] = $naf->database()->execute($sql);
                }
                

                $this->createDocumentType();
            }
			
			if(version_compare($old,'v1.6.2') == -1){
				$sql 	= include(__DIR__.'/Resource/Db/v1.6.2/Mysql.php');
				foreach($sql as $sql){
					$naf->database()->execute($sql);
				}
            }

            if(version_compare($old,'v2.1.7') == -1){
				$sql 	= include(__DIR__.'/Resource/Db/v2.1.7/Mysql.php');
				foreach($sql as $sql){
					$naf->database()->execute($sql);
				}
            }

            $this->updateNotificationProfile();
        }
    }

    public function createUser(\Neuron\Person\Entity $person, $additional = array()){

        if ($naf = NAF\Entity::get()) {

            $storage = \Neuron\Composite\User\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
            $user = new \Neuron\Composite\User($storage, $naf->base()->getModuleConfig()['actor']['user']);
            
            $result = $user->register(
                $person->code, 
                $person->first_name . ' ' . $person->last_name, 
                $additional['email'],
                null,
                self::DEFAULT_LANGUAGE, 
                Authentication\Types::TYPE_DEFAULT, 
                $additional
            );

            if($result->code == $result::CODE_SUCCESS){
                $result = $this->createRoleDefault();
                if($result->code == $result::CODE_SUCCESS){
                    $role = $result->data;
                    $result = $this->grantUser($user, $role);
                }
                $this->mappingUser($person, $user);

                $registermaster = $this->registerMaster($user);
            }else{
                $user->terminate();
            }

            return $result;

        }
    }

    public function grantUser($user, $role){
        $result = new Result();
        if ($naf = NAF\Entity::get()) {

            $storage = \Neuron\Access\Grant\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
            $grant = new \Neuron\Access\Grant($storage);

            /* grant login ke user yg baru di create */
            $result = $grant->register(
                \Neuron\Access\Grant\Type\Storage::INDIVIDUAL_ROLE,
                $role->code,
                self::DEFAULT_PERMISSION,
                $user->id,
                $storage->now(),
                null
            );

            return $result;
        }
    }

    public function createRoleDefault(){
        $result = new Result();
        if ($naf = NAF\Entity::get()) {
            $storage = \Neuron\Access\Role\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig());
            $role = new \Neuron\Access\Role($storage);
            $role->load("user",  \Neuron\Access\Role\Storage::LOADBY_CODE);
            if($role->id == null || $role->id == ''){
                $result = $role->register(
                    null, 'User', 'User', 'user'
                );
            }

            $result->data = $role;

            return $result;
        }
    }

    public function mappingUser(\Neuron\Person\Entity $person, $user){
        $data['mappingId'] = null;
        $data['personId'] = $person->id;
        $data['userId'] = $user->id;
        $data['endDate'] = null;
        $result = $person->saveMapping($data);
        return $result;
    }

    public function downloadAsset($url, $file, $path){

        $entity = NAF\Entity::get();

        if(!file_exists(getcwd() . $path)){
            mkdir(getcwd() . $path, 0775, true);
        }

        if(file_put_contents($entity->base()->getRunDir().'/temp/'.$file, @fopen($url, 'r'))){
            $zip = new \ZipArchive;
            $res = $zip->open($file);
            if ($res === TRUE) {
                $zip->extractTo(getcwd() . $path);
                $zip->close();
                //
            }
            unlink($entity->base()->getRunDir().'/temp/'.$file);
        }
    }

    public function registerMaster($user){   
        $result = new Result();
        $naf = NAF\Entity::get();
        $baseurl = $naf->layout()->getHomeURL();

        $curlCompany = curl_init();
        curl_setopt($curlCompany, CURLOPT_URL, self::BASE_URL_MASTER.''.self::API_LOAD_COMPANY);
        curl_setopt($curlCompany, CURLOPT_POSTFIELDS, 'data={"base_url_api":"'.$baseurl.'/"}');
        curl_setopt($curlCompany, CURLOPT_BINARYTRANSFER, true);
        curl_setopt($curlCompany, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curlCompany, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($curlCompany, CURLOPT_SSL_VERIFYPEER, 0);

        $jsonData = json_decode(curl_exec($curlCompany));
        curl_close($curlCompany);

        if(@$jsonData->code == 0){
            $userid = 1;
            $usermail = 'user1111@gmail.com';
            $usercode = 'username';
            /// create user
            $curlUser = curl_init();
            curl_setopt($curlUser, CURLOPT_URL, self::BASE_URL_MASTER.self::API_USER_CREATE);
            curl_setopt($curlUser, CURLOPT_POSTFIELDS, 'data={"id":null,"company_id":'.@$jsonData->data->id.',"user_id":"'.$user->id.'","email":"'.$user->email.'","user_name":"'.$user->code.'", "phone_no":null,"is_active":0}');
            curl_setopt($curlUser, CURLOPT_BINARYTRANSFER, true);
            curl_setopt($curlUser, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curlUser, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($curlUser, CURLOPT_SSL_VERIFYPEER, 0);
            $jsonUser = json_decode(curl_exec($curlUser));
            curl_close($curlUser);
            if(@$jsonUser->code == 0){
                $result->info = 'Success register user';
            }else{
                $result->code = self::CODE_FAIL;
                $result->info = 'Error register user';
            }
        }else{
            $result->code = self::CODE_FAIL;
            $result->info = 'Error load company';
        }
        return $result;
    }

    public function getCompanyId($baseurl = null){
        $naf = NAF\Entity::get();

        if(!$baseurl){
            $baseurl = $naf->layout()->getHomeURL();
        }
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://route.doorhrm.com/api/company/load-base-path",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => array('data' => '{"base_url_api":"'.$baseurl.'"}'),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        return json_decode($response);
    }

    public function updateMasterData($data = array()){
        $curl = curl_init();

        $productionRouteURL         = "https://route.doorhrm.com/api/user/update-data";
        $developmentRouteURL        = "http://neuron.me/route/public/api/user/update-data";

        curl_setopt_array($curl, array(
            CURLOPT_URL => $productionRouteURL,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => array('data' => json_encode($data)),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        return json_decode($response);
    }

    public function updateNotificationProfile(){
        $naf            = Entity::get();
        $profile        = new Profile(Profile\Storage::factory(
            $naf->getDb(),
            $naf->base()->getModuleConfig()
        ));
        $profile->load(1);
        $profile->email     = "mailer@neuronworks.co.id";
        $profile->configuration = '{"name":"Gmail","host":"smtp.gmail.com","port":587,"connection_class":"plain","connection_config":{"ssl":"tls","username":"mailer@neuronworks.co.id","password":"Neuron#1245"}}';
        $profile->save(true);
    }

    public function createDocumentType(){
        $naf            = NAF\Entity::get();
        // $naf->getDb(), $naf->base()->getModuleConfig()
        $documentType   = new \Neuron\Document\Type(
            \Neuron\Document\Type\Storage::factory(
                $naf->getDb(),
                $naf->base()->getModuleConfig()
            )
        );
        $documentTypeData = array(
            "document_type_id"  =>  null, 
            "parent_type_id"    =>  null, 
            "name"              =>  "Person", 
            "description"       =>  "Person Document", 
        );
        // $documentType->push($documentTypeData);
        // $createDocumentType = $documentType->save(false);
		
		$loadByName = $documentType->loadByName("Person");
		
		if(!$loadByName){
			$result = $documentType->save(false);
		}else{
			$result = true;
		}
    }

    public function addProfileBeforePills($template) {
        $naf = NAF\Entity::get();
        $controller = $naf->getController();

        $profileconfig = str_replace("%target%",'person-personal-content',$template);
        $profileconfig = str_replace("%icon%",'fas fa-user-circle',$profileconfig);
        $profileconfig = str_replace("%title%",$controller->translate("Personal information"),$profileconfig);

        echo $profileconfig;
    }
    public function addProfileAfterPills($template) {
        $naf = NAF\Entity::get();
        $controller = $naf->getController();

        $experience = str_replace("%target%",'person-experience-content',$template);
        $experience = str_replace("%icon%",'fas fa-id-badge',$experience);
        $experience = str_replace("%title%",$controller->translate("Job experience"),$experience);
        
        $education = str_replace("%target%",'person-education-content',$template);
        $education = str_replace("%icon%",'fas fa-user-graduate',$education);
        $education = str_replace("%title%",$controller->translate("School & Education"),$education);
        
        
        $medical = str_replace("%target%",'person-medical-content',$template);
        $medical = str_replace("%icon%",'fas fa-notes-medical',$medical);
        $medical = str_replace("%title%",$controller->translate("Medical history"),$medical);

        $contact = str_replace("%target%",'vwContact',$template);
        $contact = str_replace("%icon%",'fas fa-phone-square',$contact);
        $contact = str_replace("%title%",$controller->translate("Manage your Contact"),$contact);

        $skills  = str_replace("%target%", 'person-skill-content', $template);
        $skills  = str_replace("%icon%", 'fas fa-brain', $skills);
        $skills  = str_replace("%title%", $controller->translate("Skills"), $skills);
        
        echo $contact;
        echo $experience;
        echo $skills;
        echo $education;
        echo $medical;
    }
    
    public function addContentPersonal($user) {
        $naf                = NAF\Entity::get();
        $config             = $naf->base()->getModuleConfig();
        
        $profileController  = new \Neuron\Person\Controller\Profile();

        /* get view personal information */
        echo $profileController->ProfileAction($user);
        echo $profileController->experienceAction($user);
        echo $profileController->educationAction($user);
        echo $profileController->medicalAction($user);
        echo $profileController->contactAction($user);
        echo $profileController->skillsAction($user);
        
    }
	
	public function downloadCv($template, $user){
		$naf                = NAF\Entity::get();
        $config             = $naf->base()->getModuleConfig();
		$controller 		= $naf->getController();
		$user_id = $naf->session()->owner();
        $session = $naf->session()->get();
        
        if($user_id == $user || $session['active_role']['code'] =='hrd' || $session['active_role']['code'] == 'admin'){
            $personMap  	= new \Neuron\Person\Map(\Neuron\Person\Map\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
            $personMap		->load($user, \Neuron\Person\Map\Storage::LOADBY_USER_ID);
            $personMapData	= $personMap->pull();
            $personId		= $personMapData['person_id'];		
            
            $downloadCvUser = str_replace("%target%", $personId ,$template);
            $downloadCvUser = str_replace("%title%",$controller->translate("Download as PDF"),$downloadCvUser);
            
            echo $downloadCvUser;
        }
	}

    public function nafDoSearch($keyword, $previousResult = null) {
        $naf                = NAF\Entity::get();
        $config             = $naf->base()->getModuleConfig();
		$controller 		= $naf->getController();
        $session            = $naf->session()->get();

        $post['start']	= 0;
        $post['length']	= 3;
        $params         = array(
            'page' 		=> ($post['start'] + $post['length']) / $post['length'],
            'records' 	=> $post['length'],
        );

        $keywords 		= isset($keyword) && !empty($keyword) ? $keyword : null;

        $mEntities 		= new mEntities(mEntity\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
        $entities 		= $mEntities->load($keywords, mEntity\Storage::LISTBY_PAGE, $params);
        $rows  			= $entities->data['rows'];
        $array          = [];
        
        if ($rows) {
            foreach ($rows as $key) {
                $row['module']  = 'Person';
                $row['id']		= $key['id'];
                $row['title']	= $key['first_name'].' '.$key['last_name'];

                if (class_exists('\Neuron\Employee\Entity')) {
                    $emp	= new \Neuron\Employee\Entity(\Neuron\Employee\Entity\Storage::factory($naf->getDb(), $naf->base()->getModuleConfig()));
                    $emp->load($key['id'], \Neuron\Employee\Entity\Storage::LOADBY_PERSON);
                    $row['url']	= $emp->code ? $controller->getPublicUrl().'/actor/form/profile/'.$emp->code : null;
                } else {
                    $row['url']	= '';
                }
                $row['summary'] = isset($key['description']) ? $row['description'] : $controller->translate('this person have no description');

                $array[]	    = $row;		
            }
        }
        
        /* DUMMY -- lakukan search menggunakan model seperti komentar di atas */
        $searchResult = new \Neuron\Generic\Result();
        $searchResult->data = $array;
    
        /* PENTING: LOGIC di bawah ini harus disertakan dalam event agar hasil search dari semua modul dapat tergabung menjadi satu */
    
        /* cek apakah ada hasil search sebelumnya? */
        if (($previousResult instanceof \Neuron\Generic\Result)) {
    
            /* kalau hasil search di modul ini gagal, return hasil search sebelumnya */
            if ($searchResult->code != $searchResult::CODE_SUCCESS) return $previousResult;
    
            /* append hasil search modul ini ke hasil search sebelumnya */
            if (is_array($previousResult->data)) {
                $temp = $previousResult->data;
                foreach ($searchResult->data as $item) {
                    $temp[] = $item;
                }
                
                $previousResult->data = $temp;
                return $previousResult;
            }
    
        }
    
        /* by default return searchResult */
        return $searchResult;
    }
    
}